package com.alivc.livepush;

import static android.os.Environment.MEDIA_MOUNTED;

import android.content.Context;
import android.graphics.Bitmap;
import android.os.Environment;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.view.SurfaceView;

import androidx.annotation.NonNull;

import com.alivc.live.pusher.AlivcImageFormat;
import com.alivc.live.pusher.AlivcLiveBase;
import com.alivc.live.pusher.AlivcLivePushBGMListener;
import com.alivc.live.pusher.AlivcLivePushConfig;
import com.alivc.live.pusher.AlivcLivePushError;
import com.alivc.live.pusher.AlivcLivePushErrorListener;
import com.alivc.live.pusher.AlivcLivePushInfoListener;
import com.alivc.live.pusher.AlivcLivePushLogLevel;
import com.alivc.live.pusher.AlivcLivePushStatsInfo;
import com.alivc.live.pusher.AlivcLivePusher;
import com.alivc.live.pusher.AlivcPreviewDisplayMode;
import com.alivc.live.pusher.AlivcSnapshotListener;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

import com.alivc.live.pusher.AlivcSoundFormat;

public class LivePusher {

    private static final String TAG = "LivePusher";

    private EventChannel.EventSink mEventSink;
    private AlivcLivePusher mAlivcLivePusher;
    private AlivcLivePushConfig mAlivcLivePushConfig;

    private final Context mContext;

    private SurfaceView mSurfaceView;
    private boolean mSurfaceViewCreated = false;
    private Handler mMainHandler = new Handler(Looper.getMainLooper());

    public LivePusher(FlutterPlugin.FlutterPluginBinding flutterPluginBinding) {
        mAlivcLivePusher = new AlivcLivePusher();
        this.mContext = flutterPluginBinding.getApplicationContext();

        EventChannel livePushEventChannel = new EventChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "plugins.livepush.pusher.event");
        livePushEventChannel.setStreamHandler(new EventChannel.StreamHandler() {
            @Override
            public void onListen(Object arguments, EventChannel.EventSink events) {
                mEventSink = events;
            }

            @Override
            public void onCancel(Object arguments) {
                mEventSink = null;
            }
        });
    }

    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        switch (call.method) {
            case "initLivePusher":
                mAlivcLivePusher.init(mContext, mAlivcLivePushConfig);
                initListener();
                result.success(null);
                break;
            case "openLog":
                String isOpenLog = call.argument("arg");
                if (isOpenLog != null) {
                    openLog(Boolean.parseBoolean(isOpenLog));
                }
                result.success(null);
                break;
            //preview
            case "startPreview":
                if (mSurfaceViewCreated) {
                    mAlivcLivePusher.startPreview(mSurfaceView);
                }
                result.success(null);
                break;
            case "stopPreview":
                mAlivcLivePusher.stopPreview();
                result.success(null);
                break;
            case "setPreviewMirror_push":
                String previewMirror = call.argument("arg");
                if (previewMirror != null) {
                    mAlivcLivePusher.setPreviewMirror(previewMirror.equals("1"));
                }
                result.success(null);
                break;
            case "setPreviewDisplayMode_push":
                String previewDisplayMode = call.argument("arg");
                if (TextUtils.isEmpty(previewDisplayMode)) {
                    previewDisplayMode = "1";
                }
                assert (previewDisplayMode != null);
                switch (previewDisplayMode) {
                    case "0":
                        mAlivcLivePusher.setPreviewMode(AlivcPreviewDisplayMode.ALIVC_LIVE_PUSHER_PREVIEW_SCALE_FILL);
                        break;
                    case "1":
                        mAlivcLivePusher.setPreviewMode(AlivcPreviewDisplayMode.ALIVC_LIVE_PUSHER_PREVIEW_ASPECT_FIT);
                        break;
                    case "2":
                        mAlivcLivePusher.setPreviewMode(AlivcPreviewDisplayMode.ALIVC_LIVE_PUSHER_PREVIEW_ASPECT_FILL);
                        break;
                }
                result.success(null);
                break;
            //push
            case "startPushWithURL":
                String pushUrl = call.argument("arg");
                mAlivcLivePusher.startPush(pushUrl);
                result.success(null);
                break;
            case "stopPush":
                mAlivcLivePusher.stopPush();
                result.success(null);
                break;
            case "pause":
                mAlivcLivePusher.pause();
                result.success(null);
                break;
            case "resume":
                mAlivcLivePusher.resume();
                result.success(null);
                break;
            case "destroy":
                mAlivcLivePusher.destroy();
                result.success(null);
                break;
            case "resumeAsync":
                mAlivcLivePusher.resumeAsync();
                result.success(null);
                break;
            case "restartPush":
                mAlivcLivePusher.restartPush();
                result.success(null);
                break;
            case "restartPushAsync":
                mAlivcLivePusher.restartPushAsync();
                result.success(null);
                break;
            case "setPushMirror_push":
                String pushMirror = call.argument("arg");
                if (pushMirror != null) {
                    mAlivcLivePusher.setPushMirror(pushMirror.equals("1"));
                }
                result.success(null);
                break;
            //BGM
            case "setBGMEarsBack":
                String isOpenBGMEarsBack = call.argument("arg");
                if (isOpenBGMEarsBack != null) {
                    mAlivcLivePusher.setBGMEarsBack(isOpenBGMEarsBack.equals("1"));
                }
                result.success(null);
                break;
            case "setAudioDenoise":
                String isOpenBGMAudioDenoise = call.argument("arg");
                if (isOpenBGMAudioDenoise != null) {
                    mAlivcLivePusher.setAudioDenoise(isOpenBGMAudioDenoise.equals("1"));
                }
                result.success(null);
                break;
            case "setMute":
                String isEnableBGMMute = call.argument("arg");
                if (isEnableBGMMute != null) {
                    mAlivcLivePusher.setMute(isEnableBGMMute.equals("1"));
                }
                result.success(null);
                break;
            case "setBGMLoop":
                String isEnableBGMLoop = call.argument("arg");
                if (isEnableBGMLoop != null) {
                    mAlivcLivePusher.setBGMLoop(isEnableBGMLoop.equals("1"));
                }
                result.success(null);
                break;
            case "startBGMWithMusicPathAsync":
                String bgmPath = call.argument("arg");
                if (bgmPath != null) {
                    mAlivcLivePusher.startBGMAsync(bgmPath);
                }
                result.success(null);
                break;
            case "stopBGMAsync":
                mAlivcLivePusher.stopBGMAsync();
                result.success(null);
                break;
            case "pauseBGM":
                mAlivcLivePusher.pauseBGM();
                result.success(null);
                break;
            case "resumeBGM":
                mAlivcLivePusher.resumeBGM();
                result.success(null);
                break;
            case "setCaptureVolume":
                String captureVolume = call.argument("arg");
                if (captureVolume != null) {
                    mAlivcLivePusher.setCaptureVolume(Integer.parseInt(captureVolume));
                }
                result.success(null);
                break;
            case "setBGMVolume":
                String bgmVolume = call.argument("arg");
                if (bgmVolume != null) {
                    mAlivcLivePusher.setBGMVolume(Integer.parseInt(bgmVolume));
                }
                result.success(null);
                break;
            case "setTargetVideoBitrate_push":
                String targetVideoBitrate = call.argument("arg");
                if (targetVideoBitrate != null) {
                    mAlivcLivePusher.setTargetVideoBitrate(Integer.parseInt(targetVideoBitrate));
                }
                result.success(null);
                break;
            case "setMinVideoBitrate_push":
                String minVideoBitrate = call.argument("arg");
                if (minVideoBitrate != null) {
                    mAlivcLivePusher.setMinVideoBitrate(Integer.parseInt(minVideoBitrate));
                }
                result.success(null);
                break;
            //
            case "setFlash":
                String enableFlash = call.argument("arg");
                if (enableFlash != null) {
                    mAlivcLivePusher.setFlash(enableFlash.equals("1"));
                }
                result.success(null);
                break;
            case "switchCamera":
                mAlivcLivePusher.switchCamera();
                result.success(null);
                break;
            case "snapshot":
                Map<String, Object> snapShotParams = call.argument("arg");
                if (snapShotParams != null) {
                    String snapshotCountParams = (String) snapShotParams.get("count");
                    String snapshotIntervalParams = (String) snapShotParams.get("interval");
                    String snapshotSaveDirParams = (String) snapShotParams.get("saveDir");

                    mAlivcLivePusher.snapshot(Integer.parseInt(snapshotCountParams), Integer.parseInt(snapshotIntervalParams), new AlivcSnapshotListener() {
                        @Override
                        public void onSnapshot(Bitmap bitmap) {
                            String dateFormat = new SimpleDateFormat("yyyy-MM-dd-hh-mm-ss-SS").format(new Date());
                            File f = new File(snapshotSaveDirParams, "snapshot-" + dateFormat + ".png");
                            if (f.exists()) {
                                f.delete();
                            }
                            try {
                                FileOutputStream out = new FileOutputStream(f);
                                bitmap.compress(Bitmap.CompressFormat.PNG, 90, out);
                                out.flush();
                                out.close();
                            } catch (FileNotFoundException e) {
                                e.printStackTrace();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                            mMainHandler.post(() -> {
                                if (mEventSink != null) {
                                    Map<String, Object> snapShotMap = new HashMap<>();
                                    snapShotMap.put("method", "onSnapshot");
                                    snapShotMap.put("saveResult", "1");
                                    snapShotMap.put("savePath", snapshotSaveDirParams + "/snapshot-" + dateFormat + ".png");
                                    mEventSink.success(snapShotMap);
                                }
                            });
                            Log.e("LivePusher", "截图已保存:" + snapshotSaveDirParams + "/snapshot-" + dateFormat + ".png");
                        }
                    });
                }
                result.success(null);
                break;
            case "addWatermark":
                Map<String, Object> addWatermarkParams = call.argument("arg");
                if (addWatermarkParams != null) {
                    String addWaterMarkPath = (String) addWatermarkParams.get("path");
                    String addWaterMarkCoordX = (String) addWatermarkParams.get("coordX");
                    String addWaterMarkCoordY = (String) addWatermarkParams.get("coordY");
                    String addWaterMarkWidth = (String) addWatermarkParams.get("width");
                    mAlivcLivePusher.addWaterMark(addWaterMarkPath, Float.valueOf(addWaterMarkCoordX),
                            Float.valueOf(addWaterMarkCoordY), Float.valueOf(addWaterMarkWidth));
                }
                result.success(null);
                break;
            case "setWatermarkVisible":
                String setWatermarkVisible = call.argument("arg");
                if (setWatermarkVisible != null) {
                    mAlivcLivePusher.setWatermarkVisible(setWatermarkVisible.equals("1"));
                }
                result.success(null);
                break;
            case "sendMessage":
                Map<String, Object> sendMessage = call.argument("arg");
                if (sendMessage != null) {
                    String sendMessageMsg = (String) sendMessage.get("message");
                    String sendMessageCount = (String) sendMessage.get("count");
                    String sendMessageTime = (String) sendMessage.get("time");
                    String sendMessageIsKeyFrame = (String) sendMessage.get("isKeyFrame");
                    mAlivcLivePusher.sendMessage(sendMessageMsg, Integer.valueOf(sendMessageCount),
                            Integer.valueOf(sendMessageTime), sendMessageIsKeyFrame.equals("1"));
                }
                result.success(null);
                break;
            case "addDynamicWaterMarkImageData":
                Map<String, Object> addDynamicWaterMarkImageDataMap = call.argument("arg");
                if (addDynamicWaterMarkImageDataMap != null) {
                    String addDynamicWaterMarkImageDataMapPath = (String) addDynamicWaterMarkImageDataMap.get("path");
                    String addDynamicWaterMarkImageDataMapX = (String) addDynamicWaterMarkImageDataMap.get("x");
                    String addDynamicWaterMarkImageDataMapY = (String) addDynamicWaterMarkImageDataMap.get("y");
                    String addDynamicWaterMarkImageDataMapW = (String) addDynamicWaterMarkImageDataMap.get("w");
                    String addDynamicWaterMarkImageDataMapH = (String) addDynamicWaterMarkImageDataMap.get("h");
                    mAlivcLivePusher.addDynamicsAddons(addDynamicWaterMarkImageDataMapPath, Float.valueOf(addDynamicWaterMarkImageDataMapX),
                            Float.valueOf(addDynamicWaterMarkImageDataMapY), Float.valueOf(addDynamicWaterMarkImageDataMapW),
                            Float.valueOf(addDynamicWaterMarkImageDataMapH));
                }
                result.success(null);
                break;
            //config
            case "setExternMainStream":
                String externMainStream = call.argument("arg");
                if (externMainStream != null) {
                    mAlivcLivePushConfig.setExternMainStream(externMainStream.equals("1"), AlivcImageFormat.IMAGE_FORMAT_YUVNV12, AlivcSoundFormat.SOUND_FORMAT_S16);
                }
                result.success(null);
                break;
            case "sendVideoData":
                Map<String, Object> sendVideoDataMap = call.argument("arg");
                if (sendVideoDataMap != null) {
                    byte[] sendVideoDataMapData = (byte[]) sendVideoDataMap.get("data");
                    String sendVideoDataMapWidth = (String) sendVideoDataMap.get("width");
                    String sendVideoDataMapHeight = (String) sendVideoDataMap.get("height");
                    String sendVideoDataMapSize = (String) sendVideoDataMap.get("size");
                    String sendVideoDataMapPts = (String) sendVideoDataMap.get("pts");
                    mAlivcLivePusher.inputStreamVideoData(sendVideoDataMapData, Integer.parseInt(sendVideoDataMapWidth),
                            Integer.parseInt(sendVideoDataMapHeight), Integer.parseInt(sendVideoDataMapWidth),
                            Integer.parseInt(sendVideoDataMapSize), Long.parseLong(sendVideoDataMapPts), 0);
                }
                break;
            case "sendPCMData":
                Map<String, Object> sendPCMDataMap = call.argument("arg");
                if (sendPCMDataMap != null) {
                    byte[] sendPCMDataMapData = (byte[]) sendPCMDataMap.get("data");
                    String sendPCMDataMapSize = (String) sendPCMDataMap.get("size");
                    String sendPCMDataMapSampleRate = (String) sendPCMDataMap.get("sampleRate");
                    String sendPCMDataMapChannel = (String) sendPCMDataMap.get("channel");
                    String sendPCMDataMapPts = (String) sendPCMDataMap.get("pts");
                    mAlivcLivePusher.inputStreamAudioData(sendPCMDataMapData, Integer.parseInt(sendPCMDataMapSize),
                            Integer.parseInt(sendPCMDataMapSampleRate), Integer.parseInt(sendPCMDataMapChannel),
                            Long.parseLong(sendPCMDataMapPts));
                }
                break;
        }
    }

    public void openLog(boolean isOpen) {
        AlivcLiveBase.setLogLevel(AlivcLivePushLogLevel.AlivcLivePushLogLevelInfo);
        AlivcLiveBase.setConsoleEnabled(isOpen);
        if (isOpen) {
            // adb pull /sdcard/Android/data/com.alivc.live.pusher.demo/files/Ali_RTS_Log/ ~/Downloads
            String logPath = getLogFilePath(mContext.getApplicationContext(), null);
            // full log file limited was kLogMaxFileSizeInKB * 5 (parts)
            int maxPartFileSizeInKB = 100 * 1024 * 1024; //100G
            AlivcLiveBase.setLogDirPath(logPath, maxPartFileSizeInKB);
        }
    }

    private static String getLogFilePath(@NonNull Context context, String dir) {
        String logFilePath = "";
        //判断SD卡是否可用
        if (MEDIA_MOUNTED.equals(Environment.getExternalStorageState())) {
            logFilePath = context.getExternalFilesDir(dir).getAbsolutePath();
        } else {
            //没内存卡就存机身内存
            logFilePath = context.getFilesDir() + File.separator + dir;
        }
        File file = new File(logFilePath);
        if (!file.exists()) {//判断文件目录是否存在
            file.mkdirs();
        }

        Log.d(TAG, "log file path: " + logFilePath);
        return logFilePath;
    }


    public void setPreviewView(LivePusherView livePusherView) {
        livePusherView.setOnSurfaceHolderCallback(new LivePusherView.OnSurfaceHolderCallback() {
            @Override
            public void surfaceCreated() {
                mSurfaceViewCreated = true;
                mSurfaceView = (SurfaceView) livePusherView.getView();
            }

            @Override
            public void surfaceChanged() {

            }

            @Override
            public void surfaceDestroyed() {
                mSurfaceViewCreated = false;
            }
        });
    }

    private void initListener() {
        mAlivcLivePusher.setLivePushBGMListener(new AlivcLivePushBGMListener() {
            @Override
            public void onStarted() {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMStarted");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onStopped() {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMStoped");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onPaused() {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMPaused");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onResumed() {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMResumed");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onProgress(long progress, long duration) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMProgress");
                        result.put("progress", String.valueOf(progress));
                        result.put("duration", String.valueOf(duration));
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onCompleted() {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMCompleted");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onDownloadTimeout() {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMDownloadTimeout");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onOpenFailed() {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onBGMOpenFailed");
                        mEventSink.success(result);
                    }
                });
            }
        });

        mAlivcLivePusher.setLivePushErrorListener(new AlivcLivePushErrorListener() {
            @Override
            public void onSystemError(AlivcLivePusher alivcLivePusher, AlivcLivePushError alivcLivePushError) {
                Map<String, Object> result = new HashMap<>();
                result.put("method", "onSystemError");
                result.put("errorCode", alivcLivePushError.getCode());
                result.put("errorDescription", alivcLivePushError.getMsg());
                mEventSink.success(result);
            }

            @Override
            public void onSDKError(AlivcLivePusher alivcLivePusher, AlivcLivePushError alivcLivePushError) {
                Map<String, Object> result = new HashMap<>();
                result.put("method", "onSDKError");
                result.put("errorCode", alivcLivePushError.getCode());
                result.put("errorDescription", alivcLivePushError.getMsg());
                mEventSink.success(result);
            }
        });

        mAlivcLivePusher.setLivePushInfoListener(new AlivcLivePushInfoListener() {
            @Override
            public void onPreviewStarted(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onPreviewStarted");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onPreviewStopped(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onPreviewStoped");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onPushStarted(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onPushStarted");
                        mEventSink.success(result);
                    }
                });

            }

            @Override
            public void onPushPaused(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onPushPaused");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onPushResumed(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onPushResumed");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onPushStopped(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onPushStoped");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onPushRestarted(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onPushRestart");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onFirstFramePreviewed(AlivcLivePusher alivcLivePusher) {
                mMainHandler.post(() -> {
                    if (mEventSink != null) {
                        Map<String, Object> result = new HashMap<>();
                        result.put("method", "onFirstFramePreviewed");
                        mEventSink.success(result);
                    }
                });
            }

            @Override
            public void onFirstFramePushed(AlivcLivePusher alivcLivePusher) {
                if (mEventSink != null) {

                }
//                Map<String,Object> result = new HashMap<>();
//                result.put("method","onFirstFramePushed");
//                mEventSink.success(result);
            }

            @Override
            public void onDropFrame(AlivcLivePusher alivcLivePusher, int i, int i1) {
                if (mEventSink != null) {

                }
//                Map<String, Object> result = new HashMap<>();
//                result.put("method", "onFirstFramePushed");
//                mEventSink.success(result);
            }

            @Override
            public void onAdjustBitrate(AlivcLivePusher alivcLivePusher, int i, int i1) {
                if (mEventSink != null) {

                }

            }

            @Override
            public void onAdjustFps(AlivcLivePusher alivcLivePusher, int i, int i1) {
                if (mEventSink != null) {

                }
            }

            @Override
            public void onPushStatistics(AlivcLivePusher alivcLivePusher, AlivcLivePushStatsInfo alivcLivePushStatsInfo) {

            }

            @Override
            public void onSetLiveMixTranscodingConfig(AlivcLivePusher alivcLivePusher, boolean b, String s) {

            }
        });
    }

    public void setPushConfig(LivePushConfig livePushConfig) {
        this.mAlivcLivePushConfig = livePushConfig.getPushConfig();
    }
}
