package com.alivc.livepush;

import android.content.Context;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;

import androidx.annotation.NonNull;

import io.flutter.plugin.platform.PlatformView;


public class LivePusherView implements PlatformView {

    private final SurfaceView mSurfaceView;
    private OnSurfaceHolderCallback mOnSurfaceHolderCallback;

    public LivePusherView(Context context) {
        mSurfaceView = new SurfaceView(context);
        mSurfaceView.getHolder().addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(@NonNull SurfaceHolder surfaceHolder) {
                if (mOnSurfaceHolderCallback != null) {
                    mOnSurfaceHolderCallback.surfaceCreated();
                }
            }

            @Override
            public void surfaceChanged(@NonNull SurfaceHolder surfaceHolder, int i, int i1, int i2) {
                if (mOnSurfaceHolderCallback != null) {
                    mOnSurfaceHolderCallback.surfaceChanged();
                }
            }

            @Override
            public void surfaceDestroyed(@NonNull SurfaceHolder surfaceHolder) {
                if (mOnSurfaceHolderCallback != null) {
                    mOnSurfaceHolderCallback.surfaceDestroyed();
                }
            }
        });
    }

    @Override
    public View getView() {
        return mSurfaceView;
    }

    @Override
    public void dispose() {
    }

    public interface OnSurfaceHolderCallback {
        void surfaceCreated();
        void surfaceChanged();
        void surfaceDestroyed();
    }

    public void setOnSurfaceHolderCallback(OnSurfaceHolderCallback callback) {
        this.mOnSurfaceHolderCallback = callback;
    }
}